﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace PhysicTest1
{
    public class Camera
    {
        
        private Viewport _Viewport;
        private Vector3 _Position;
        private Matrix _ViewMatrix;
        private Matrix _ProjMatrix;

        #region Properties
        
      
        public float Pitch
        {
            get;
            private set;
        }
        public float Yaw
        {
            get;
            private set;
        }
        public Vector3 CameraLookAt
        {
            get;
            private set;
        }
        public Matrix WorldMatrix
        {
            get;
            private set;
        }
        public Point MousePosition
        {
            get;
            private set;
        }
       

        public Viewport Viewport
        {
            get
            {
                return _Viewport;
            }
            set
            {
                ProjectionMatrix = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, value.AspectRatio, 1.0f, 10000.0f);
                _Viewport = value;
            }
        }
        public Vector3 Position
        {
            get
            {
                return _Position;
            }
            set
            {
                _ViewMatrix.Translation = value;
                _Position = value;
            }
        }
        public Matrix ViewMatrix
        {
            get
            {
                return _ViewMatrix;
            }
            set
            {
                _ViewMatrix = value;
            }
        }
        public Matrix ProjectionMatrix
        {
            get
            {
                return _ProjMatrix;
            }
            set
            {
                _ProjMatrix = value;
            }
        }
        #endregion

        

        public Camera(Vector3 position,Vector2 YawPitch,Viewport viewport)
        {
            _Viewport = viewport;
            _Position = position;
            Matrix cameraRotation = Matrix.CreateFromYawPitchRoll(YawPitch.X, YawPitch.Y, 0.0f);
            CameraLookAt = Position + Vector3.TransformNormal(Vector3.Forward, cameraRotation);
            
            this.ViewMatrix = Matrix.CreateLookAt(Position, CameraLookAt, new Vector3(0, 1, 0));
            this.ProjectionMatrix = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, Viewport.AspectRatio, 0.1f, 10000.0f);

            CenterCursor();
        }

        public virtual void Update(GameTime elapsedTime)
        {

            Vector2 cursorPosition = new Vector2(Mouse.GetState().X, Mouse.GetState().Y);
            Vector2 mouseCenter = new Vector2(Viewport.Width / 2, Viewport.Height / 2);
            Vector2 delta = cursorPosition - mouseCenter;
            Vector2 deltaDampened = delta * 0.001f;
            CenterCursor();

            // Should perhaps extract the yaw and pitch from the current direction of the camera
            Yaw -= deltaDampened.X;
            Pitch -= deltaDampened.Y;

            
            WorldMatrix = Matrix.CreateFromAxisAngle(Vector3.Right, Pitch) * Matrix.CreateFromAxisAngle(Vector3.Up, Yaw);


            Position = Matrix.Invert(this.ViewMatrix).Translation;

            Matrix cameraRotation = Matrix.CreateFromYawPitchRoll(Yaw, Pitch, 0.0f);
            Vector3 newForward = Vector3.TransformNormal(Vector3.Forward, cameraRotation);

            float elapsed = (float)(elapsedTime.ElapsedGameTime.TotalMilliseconds / 1000.0); // Elapsed time since last frame in seconds
            const float speed = 20.0f; // 20 distance units per second
            float distance = speed * elapsed; // d = vt

            // The amount of movement * the direction of movement, then rotate that along the direction we are looking
            Vector3 translateDirection = Vector3.Zero;

            KeyboardState states = Keyboard.GetState();

            if (states.IsKeyDown(Keys.W)) // Forwards
                translateDirection += Vector3.TransformNormal(Vector3.Forward, cameraRotation);

            if (states.IsKeyDown(Keys.S)) // Backwards
                translateDirection += Vector3.TransformNormal(Vector3.Backward, cameraRotation);

            if (states.IsKeyDown(Keys.A)) // Left
                translateDirection += Vector3.TransformNormal(Vector3.Left, cameraRotation);

            if (states.IsKeyDown(Keys.D)) // Right
                translateDirection += Vector3.TransformNormal(Vector3.Right, cameraRotation);

            Vector3 newPosition = Position;
            if (translateDirection.LengthSquared() > 0)
                newPosition += Vector3.Normalize(translateDirection) * distance;
            CameraLookAt = newPosition + newForward;
            this.ViewMatrix = Matrix.CreateLookAt(newPosition, CameraLookAt, Vector3.Up);
        }

      

        private void CenterCursor()
        {
            MousePosition = new Point(Viewport.Width / 2, Viewport.Height / 2);
            Mouse.SetPosition(MousePosition.X, MousePosition.Y);
        }
 
    }
}
