﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using BEPUphysics.Entities;

namespace PhysicTest1
{
    /// <remarks>
    /// Simple display class for boxes.  Creates and manages a triangulated represenation of the entity.
    /// </remarks>
    public class DisplayedBox : DisplayedPrimitive
    {
        /// <summary>
        /// Creates the display object for the entity.
        /// </summary>
        /// <param name="graphics">Manager to use.</param>
        /// <param name="initShape">Whether or not to initialize the local shape vertices.</param>
        /// <param name="initVertexBuffer">Whether or not to initialize the local vertex buffer.</param>
        public DisplayedBox(Box body,Color diffusecolor, bool initShape, bool initVertexBuffer) :
            base(body,diffusecolor)
        {
            
            textureCoords[0] = new Vector2(0, 0);
            textureCoords[1] = new Vector2(1, 0);
            textureCoords[2] = new Vector2(1, 1);
            textureCoords[3] = new Vector2(0, 1);

            if (initShape || initVertexBuffer)
                initializeShape(new Vector3(body.Width + body.CollisionMargin * 2, body.Height + body.CollisionMargin * 2, body.Length + body.CollisionMargin * 2));
            if (initVertexBuffer)
            {
                vertexBuffer = new VertexBuffer(myGraphics.GraphicsDevice, typeof(VertexPositionNormalTexture), shapeVertices.Length, BufferUsage.WriteOnly);
                vertexBuffer.SetData<VertexPositionNormalTexture>(shapeVertices);
            }

        }

        public Vector3 GetBBSize(BoundingBox box)
        {
            Vector3 result = box.Max - box.Min;
            //result.X -= 0.12f;
            //result.Y -= 0.12f;
            //result.Z -= 0.12f;
            return result;
        }

        void initializeShape(Vector3 size)
        {
       
            Vector3 sz = size;
            //sz.X -= Body.CollisionMargin;
            //sz.Y -= Body.CollisionMargin;
            //sz.Z -= Body.CollisionMargin;
            BoundingBox boundingBox = new BoundingBox(-sz/2, sz/2);

            Vector3[] corners = boundingBox.GetCorners();
            shapeVertices = new VertexPositionNormalTexture[36];

            shapeVertices[0] = new VertexPositionNormalTexture(corners[0], Vector3.Backward, textureCoords[0]);
            shapeVertices[1] = new VertexPositionNormalTexture(corners[1], Vector3.Backward, textureCoords[1]);
            shapeVertices[2] = new VertexPositionNormalTexture(corners[2], Vector3.Backward, textureCoords[2]);
            shapeVertices[3] = new VertexPositionNormalTexture(corners[0], Vector3.Backward, textureCoords[0]);
            shapeVertices[4] = new VertexPositionNormalTexture(corners[2], Vector3.Backward, textureCoords[2]);
            shapeVertices[5] = new VertexPositionNormalTexture(corners[3], Vector3.Backward, textureCoords[3]);

            shapeVertices[6] = new VertexPositionNormalTexture(corners[1], Vector3.Right, textureCoords[0]);
            shapeVertices[7] = new VertexPositionNormalTexture(corners[5], Vector3.Right, textureCoords[1]);
            shapeVertices[8] = new VertexPositionNormalTexture(corners[6], Vector3.Right, textureCoords[2]);
            shapeVertices[9] = new VertexPositionNormalTexture(corners[1], Vector3.Right, textureCoords[0]);
            shapeVertices[10] = new VertexPositionNormalTexture(corners[6], Vector3.Right, textureCoords[2]);
            shapeVertices[11] = new VertexPositionNormalTexture(corners[2], Vector3.Right, textureCoords[3]);

            shapeVertices[12] = new VertexPositionNormalTexture(corners[5], Vector3.Forward, textureCoords[0]);
            shapeVertices[13] = new VertexPositionNormalTexture(corners[4], Vector3.Forward, textureCoords[1]);
            shapeVertices[14] = new VertexPositionNormalTexture(corners[7], Vector3.Forward, textureCoords[2]);
            shapeVertices[15] = new VertexPositionNormalTexture(corners[5], Vector3.Forward, textureCoords[0]);
            shapeVertices[16] = new VertexPositionNormalTexture(corners[7], Vector3.Forward, textureCoords[2]);
            shapeVertices[17] = new VertexPositionNormalTexture(corners[6], Vector3.Forward, textureCoords[3]);

            shapeVertices[18] = new VertexPositionNormalTexture(corners[4], Vector3.Left, textureCoords[0]);
            shapeVertices[19] = new VertexPositionNormalTexture(corners[0], Vector3.Left, textureCoords[1]);
            shapeVertices[20] = new VertexPositionNormalTexture(corners[3], Vector3.Left, textureCoords[2]);
            shapeVertices[21] = new VertexPositionNormalTexture(corners[4], Vector3.Left, textureCoords[0]);
            shapeVertices[22] = new VertexPositionNormalTexture(corners[3], Vector3.Left, textureCoords[2]);
            shapeVertices[23] = new VertexPositionNormalTexture(corners[7], Vector3.Left, textureCoords[3]);

            shapeVertices[24] = new VertexPositionNormalTexture(corners[0], Vector3.Up, textureCoords[2]);
            shapeVertices[25] = new VertexPositionNormalTexture(corners[5], Vector3.Up, textureCoords[0]);
            shapeVertices[26] = new VertexPositionNormalTexture(corners[1], Vector3.Up, textureCoords[3]);
            shapeVertices[27] = new VertexPositionNormalTexture(corners[0], Vector3.Up, textureCoords[2]);
            shapeVertices[28] = new VertexPositionNormalTexture(corners[4], Vector3.Up, textureCoords[1]);
            shapeVertices[29] = new VertexPositionNormalTexture(corners[5], Vector3.Up, textureCoords[0]);

            shapeVertices[30] = new VertexPositionNormalTexture(corners[3], Vector3.Down, textureCoords[0]);
            shapeVertices[31] = new VertexPositionNormalTexture(corners[2], Vector3.Down, textureCoords[1]);
            shapeVertices[32] = new VertexPositionNormalTexture(corners[6], Vector3.Down, textureCoords[2]);
            shapeVertices[33] = new VertexPositionNormalTexture(corners[3], Vector3.Down, textureCoords[0]);
            shapeVertices[34] = new VertexPositionNormalTexture(corners[6], Vector3.Down, textureCoords[2]);
            shapeVertices[35] = new VertexPositionNormalTexture(corners[7], Vector3.Down, textureCoords[3]);

        }

    }
}
