﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using BEPUphysics.Entities;

namespace PhysicTest1
{
    /// <remarks>
    /// Superclass for the various display object types.
    /// </remarks>
    public abstract class DisplayedPrimitive
    {
        public Entity Body;
        internal VertexPositionNormalTexture[] shapeVertices;
        internal VertexBuffer vertexBuffer;
        public static GraphicsDeviceManager myGraphics;
        public Color DiffuseColor;
        /// <summary>
        /// Texture to use for noninstanced and CPU instaced drawing.
        /// </summary>
        public Texture2D texture;

        /// <summary>
        /// Texture cooordinate array reused to slap on textures.
        /// </summary>
        protected Vector2[] textureCoords = new Vector2[4];


        public DisplayedPrimitive(Entity body,Color diffusecolor)
        {
            this.Body = body;
            this.DiffuseColor = diffusecolor;
        }

        /// <summary>
        /// Draws the display object.
        /// </summary>
        /// <param name="effect">Effect used to draw.</param>
        public void Draw(Effect effect, Camera camera)
        {
           
            if (effect is BasicEffect)
            {
                ((BasicEffect)effect).World = Body.WorldTransform;
                if (((BasicEffect)effect).TextureEnabled)
                    ((BasicEffect)effect).Texture = texture;
                ((BasicEffect)effect).View = camera.ViewMatrix;
                ((BasicEffect)effect).Projection = camera.ProjectionMatrix;
                ((BasicEffect)effect).DiffuseColor = (Vector3)Body.Tag;
                ((BasicEffect)effect).EnableDefaultLighting();
            }
            myGraphics.GraphicsDevice.SetVertexBuffer(vertexBuffer);

            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Apply();

                myGraphics.GraphicsDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, shapeVertices.Length / 3);

            }
        }
    }
}
