﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using BEPUphysics;
using BEPUphysics.Entities;
using BEPUphysics.ResourceManagement;

namespace PhysicTest1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Weapon weapon;
        BasicEffect beffect;
        BasicEffect bbeffect;
        public static Camera camera;
        Space scene;
        ModelDrawer drawer;
        public static Entity testentity;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            DisplayedPrimitive.myGraphics = graphics;

            camera = new Camera(new Vector3(4, .5f, 10), Vector2.Zero, GraphicsDevice.Viewport);
            weapon = new Weapon(GraphicsDevice, camera);
            drawer = new ModelDrawer();


            scene = new Space();
            scene.SimulationSettings.MotionUpdate.Gravity = new Vector3(0, -9.81f, 0f);

            if (Environment.ProcessorCount > 1)
            {
                for (int i = 0; i < Environment.ProcessorCount; i++)
                {
                    scene.ThreadManager.AddThread();
                }
            }
            scene.UseMultithreadedUpdate = true;

            AddBox(new Vector3(0, -.5f, 0), new Vector3(0, 0, 0), new Vector3(100, 1, 100), 0.0f, Color.SlateGray);
            testentity = AddBox(new Vector3(0, 1.5f, 0), new Vector3(0, 0, 0), new Vector3(1, 1, 1), 1.0f, Color.Violet);

            AddBox(new Vector3(0, 3.5f, 0), new Vector3(0, 0, 0), new Vector3(1, 1, 1), 1.0f, Color.Violet);
            AddBox(new Vector3(0, 6.5f, 0), new Vector3(0, 0, 0), new Vector3(1, 1, 1), 1.0f, Color.Violet);
            AddBox(new Vector3(0, 9.5f, 0), new Vector3(0, 0, 0), new Vector3(1, 1, 1), 1.0f, Color.Violet);

            screenshot = new RenderTarget2D(GraphicsDevice, GraphicsDevice.PresentationParameters.BackBufferWidth, GraphicsDevice.PresentationParameters.BackBufferHeight,false, SurfaceFormat.Color, DepthFormat.Depth24Stencil8);
            base.Initialize();
        }

        private Entity AddBox(Vector3 position,Vector3 rotation,Vector3 size,float mass,Color diffuse)
        {
            Box entity;
            if (mass <= 0.0f)
            {
                entity = new Box(position, size.X, size.Y, size.Z);
                entity.IsAffectedByGravity = false;
            }
            else
                entity = new Box(position, size.X, size.Y, size.Z, mass);

            scene.Add(entity);
            entity.Tag =  diffuse.ToVector3();
            drawer.Add(new DisplayedBox(entity, Color.White, true, true));
            return entity;
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            beffect = new BasicEffect(GraphicsDevice);
            bbeffect = new BasicEffect(GraphicsDevice);

            weapon.LoadContent(Content);
            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        public static Vector3 GetOriginVector()
        {
            return new Vector3(camera.Position.X, camera.Position.Y, camera.Position.Z);
        }

        public static Vector3 GetDirVector()
        {
            
            return camera.WorldMatrix.Forward;
        }


        RenderTarget2D screenshot;
        bool needscreenshot = false;
        KeyboardState lastkstate;

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            scene.Update(gameTime);
            if (lastkstate.IsKeyDown(Keys.Escape))
                Exit();
            KeyboardState kstate = Keyboard.GetState();
            if (lastkstate.IsKeyUp(Keys.Q) && kstate.IsKeyDown(Keys.Q))
            {
                needscreenshot = true;
            }

            // TODO: Add your update logic here
            MouseState mstate = Mouse.GetState();
            if (mstate.LeftButton == ButtonState.Pressed)
            {
                weapon.Shoot(scene);
            }

            camera.Update(gameTime);
            weapon.Update(gameTime);


            List<Vector3> hitLocations = Resources.GetVectorList(),
                    hitNormals = Resources.GetVectorList();
            List<float> hitTimes = Resources.GetFloatList();
            List<Entity> hitEntities = Resources.GetEntityList();

            
            Vector3 nearsource = new Vector3((float)mstate.X, (float)mstate.Y, 0f);
            Vector3 farsource = new Vector3((float)mstate.X, (float)mstate.Y, 1f);


            Vector3 nearPoint = GraphicsDevice.Viewport.Unproject(nearsource,
                camera.ProjectionMatrix, camera.ViewMatrix, Matrix.Identity);

            Vector3 farPoint = GraphicsDevice.Viewport.Unproject(farsource,
                camera.ProjectionMatrix, camera.ViewMatrix, Matrix.Identity);

            // Create a ray from the near clip plane to the far clip plane.
            Vector3 dir = farPoint - nearPoint;
            dir.Normalize();
            Ray ray = new Ray(nearPoint, dir);


            Vector3 origin = GetOriginVector();
            Vector3 direction = GetDirVector();

            if (scene.BroadPhase.RayCast(origin, direction, 1000, true, hitEntities, hitLocations, hitNormals, hitTimes))
            {
                float earliestTime = float.PositiveInfinity;
                int earliestHitIndex = 0;
                for (int k = 0; k < hitTimes.Count; k++)
                {
                    if (hitEntities[k] != null && hitEntities[k].CollisionRules.Personal > CollisionRule.Normal)
                    {
                        //Don't bother selecting intangible/detector objects.
                        hitTimes[k] = float.PositiveInfinity;
                    }
                    else if (hitTimes[k] < earliestTime)
                    {
                        earliestTime = hitTimes[k];
                        earliestHitIndex = k;
                    }
                }
                //valid ray hit
                if (hitEntities[earliestHitIndex] != null && hitTimes[earliestHitIndex] != float.PositiveInfinity && hitEntities[earliestHitIndex].IsDynamic)
                {
                    testentity.Tag = Color.Red.ToVector3();
                }
                else
                {
                    testentity.Tag = Color.Violet.ToVector3();
                }
            }
            else
            {
                testentity.Tag = Color.Violet.ToVector3();
            }

            lastkstate = kstate;
            base.Update(gameTime);
        }


        bool DisplayBoundingBoxes = true;


        void DrawBoundingBoxes()
        {
            bbeffect.LightingEnabled = false;
            bbeffect.VertexColorEnabled = true;
            bbeffect.World = Matrix.Identity;
            bbeffect.View = camera.ViewMatrix;
            bbeffect.Projection = camera.ProjectionMatrix;
            bbeffect.DiffuseColor = Color.White.ToVector3();

            if (scene.Entities.Count > 0)
            {
                List<VertexPositionColor> boundingBoxLines = new List<VertexPositionColor>();

                foreach (Entity e in scene.Entities)
                {
                    Vector3[] boundingBoxCorners = e.BoundingBox.GetCorners();
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[0], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[1], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[0], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[3], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[0], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[4], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[1], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[2], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[1], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[5], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[2], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[3], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[2], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[6], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[3], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[7], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[4], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[5], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[4], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[7], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[5], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[6], Color.DarkRed));

                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[6], Color.DarkRed));
                    boundingBoxLines.Add(new VertexPositionColor(boundingBoxCorners[7], Color.DarkRed));
                }


                foreach (EffectPass pass in bbeffect.CurrentTechnique.Passes)
                {
                    pass.Apply();
                    GraphicsDevice.DrawUserPrimitives(PrimitiveType.LineList, boundingBoxLines.ToArray(), 0, scene.Entities.Count * 12);
                }
            }
        }

        public void RenderScene(bool drawscreenshot)
        {
            drawer.Draw(beffect, camera);
            DrawBoundingBoxes();
           //weapon.Draw(beffect,drawscreenshot);

            DepthStencilState dstate = GraphicsDevice.DepthStencilState;

            spriteBatch.Begin();
            weapon.DrawCrosshair(spriteBatch);
            spriteBatch.End();

            GraphicsDevice.DepthStencilState = dstate;
        }


        int screenshotcount = 0;

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);


            if (needscreenshot)
            {
                GraphicsDevice.SetRenderTarget(screenshot);
                RenderScene(true);
                GraphicsDevice.SetRenderTarget(null);

                System.IO.FileStream fs = new System.IO.FileStream("scr"+screenshotcount.ToString(), System.IO.FileMode.OpenOrCreate);

                screenshot.SaveAsPng(fs,screenshot.Width,screenshot.Height);
                screenshotcount++;
                needscreenshot = false;
            }


            RenderScene(false);

            base.Draw(gameTime);
        }
    }
}
