﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using BEPUphysics;
using BEPUphysics.Entities;
using BEPUphysics.ResourceManagement;

namespace PhysicTest1
{
 
    public class Weapon
    {
        public struct WeaponResource
        {
           public Model model;
           public SoundEffect firesound;
           public Texture2D flashtexture;
           public Texture2D crosshairtexture;
           public string name;
        }

        List<WeaponResource> weaponResources;
        WeaponResource currentWeaponResource;
 
        public List<Vector3> bufferdiffusecolor = new List<Vector3>();
        float wpOffset = 15.0f;

        VertexPositionTexture[] quadvertexes;
        short[] quadindices;
        BasicEffect quadeffect;

        public GraphicsDevice GraphicsDevice
        {
            get;
            set;
        }

        public BasicEffect Effect
        {
            get;
            set;
        }

       

        Camera Camera
        {
            get;
            set;
        }

 

        public float IntervalBetweenShoots
        {
            get;
            set;
        }

        public float CurrentTimeBetweenShoots
        {
            get;
            set;
        }

        public bool weaponFireAnimStarted
        {
            get;
            set;
        }

 
       
        public Weapon(GraphicsDevice device,Camera camera)
        {
            this.GraphicsDevice = device;
            this.Camera = camera;
            this.CurrentTimeBetweenShoots = 0.0f;
            this.IntervalBetweenShoots = 0.33f;

            weaponResources = new List<WeaponResource>();
        }

        public void LoadContent(ContentManager content)
        {
            weaponResources.Add(new WeaponResource()
            {
                //firesound = content.Load<SoundEffect>("Colt//coltfire"),
                crosshairtexture = content.Load<Texture2D>("Colt//coltcrosshair"),
                //flashtexture = content.Load<Texture2D>("Colt//coltfireflash"),
                //model = content.Load<Model>("Colt//colt"),
                name = "Colt"
            });

            currentWeaponResource = weaponResources.Last();
            /*
            foreach (ModelMesh mesh in currentWeaponResource.model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    bufferdiffusecolor.Add(((BasicEffect)part.Effect).DiffuseColor);
                }
            }*/

            quadvertexes = new VertexPositionTexture[4];
            quadvertexes[0] = new VertexPositionTexture(new Vector3(0, 0, 0), new Vector2(0, 0));
            quadvertexes[1] = new VertexPositionTexture(new Vector3(0, 1, 0), new Vector2(0, 1));
            quadvertexes[2] = new VertexPositionTexture(new Vector3(1, 0, 0), new Vector2(1, 0));
            quadvertexes[3] = new VertexPositionTexture(new Vector3(1, 1, 0), new Vector2(1, 1));




            quadindices = new short[6]
            {
                0,1,2,
                3,2,1
            };
            quadeffect = new BasicEffect(this.GraphicsDevice);
        }


        
 
       

        public void Shoot(Space world)
        {
            if (weaponFireAnimStarted == false)
            {
                weaponFireAnimStarted = true;
                //currentWeaponResource.firesound.Play();

                List<Vector3> hitLocations = Resources.GetVectorList(), 
                    hitNormals = Resources.GetVectorList();
                List<float> hitTimes = Resources.GetFloatList();
                List<Entity> hitEntities = Resources.GetEntityList();

                Vector3 origin = Camera.Position;
                Vector3 direction = Camera.WorldMatrix.Forward; // WorldMatrix = Matrix.CreateFromAxisAngle(Vector3.Right, Pitch) * Matrix.CreateFromAxisAngle(Vector3.Up, Yaw);

                if (world.BroadPhase.RayCast(origin,direction, 1000, true, hitEntities, hitLocations, hitNormals, hitTimes))
                {
                    float earliestTime = float.PositiveInfinity;
                    int earliestHitIndex = 0;
                    for (int k = 0; k < hitTimes.Count; k++)
                    {
                        if (hitEntities[k] != null && hitEntities[k].CollisionRules.Personal > CollisionRule.Normal)
                        {
                            //Don't bother selecting intangible/detector objects.
                            hitTimes[k] = float.PositiveInfinity;
                        }
                        else if (hitTimes[k] < earliestTime)
                        {
                            earliestTime = hitTimes[k];
                            earliestHitIndex = k;
                        }
                    }
                    //valid ray hit
                    if (hitEntities[earliestHitIndex] != null && hitTimes[earliestHitIndex] != float.PositiveInfinity && hitEntities[earliestHitIndex].IsDynamic)
                    {
                        //Apply the force.
                        Entity body = hitEntities[earliestHitIndex];
                        float force = 8.0f;
                        body.ApplyImpulse(hitLocations[earliestHitIndex], Vector3.Normalize(Camera.WorldMatrix.Forward)*force);
                    }
                }
            }
        }

        /*
        Entity body = hitEntities[earliestHitIndex];
        Vector3 localOffset = Vector3.Transform(hitLocations[earliestHitIndex] - body.InternalCenterOfMass, Quaternion.Conjugate(body.InternalOrientationQuaternion));
        Vector3 position = Vector3.TransformNormal(localOffset, body.InternalOrientationMatrix) + body.InternalCenterOfMass;
        Vector3 dir = hitLocations[earliestHitIndex] - position;

         * */

        float flashCoof = 0.0f;

        public void Update(GameTime gt)
        {
            if (weaponFireAnimStarted)
            {
                this.CurrentTimeBetweenShoots += (float)gt.ElapsedGameTime.TotalSeconds;
                if(flashCoof < 0.01f) flashCoof += 0.005f;
                if (this.CurrentTimeBetweenShoots >= this.IntervalBetweenShoots)
                {
                    CurrentTimeBetweenShoots = 0.0f;
                    flashCoof = 0.0f;
                    weaponFireAnimStarted = false;
                }
            }
        }

       
        public void DrawCrosshair(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(currentWeaponResource.crosshairtexture, new Vector2(Camera.MousePosition.X-16,
                Camera.MousePosition.Y-16), Color.White);
        }
   


    }
}
